class CustomException(Exception):

  def __init__(self, message):
    super().__init__(message)
    self.__message = message

  def __str__(self):
    return str(self.__message)

class CannotLoadLocalFile(CustomException):

  def __init__(self):
    super().__init__("Cannot load local tfstate")

class CannotReadLocalFile(CustomException):

  def __init__(self):
    super().__init__("Cannot read local tfstate")

class CannotLoadRemoteFile(CustomException):

  def __init__(self):
    super().__init__("Cannot load remote tfstate")



from tflens.model.tfstate import TfState
from tflens.service.tfstate import (
  RemoteS3TfStateService,
  LocalTfStateService
)
from tflens.helper.table import (
  MarkdownTableHelper,
  HtmlTableHelper
)

class TfStateController():

  def __init__(self, tfstate_content: dict):
    self.__tfstate = TfState(
      content=tfstate_content
    )

  def show_resources(self):
    table = MarkdownTableHelper(
      rows=self.__tfstate.get_resources()
    )

    table.print()

  def create_markdown_file(self):
    table = MarkdownTableHelper(
      rows=self.__tfstate.get_resources()
    )

    table.write_markdown_file()

  def create_html_file(self):
    table = HtmlTableHelper(
      rows=self.__tfstate.get_resources()
    )

    table.write_html_file()

class LocalTfStateController(TfStateController):

  def __init__(self, file_location: str):
    self.__local_tfstate_service = LocalTfStateService(
      file_location=file_location
    )

    super().__init__(
      tfstate_content=self.__local_tfstate_service.read_content()
    )

class RemoteS3TfStateController(TfStateController):

  def __init__(self, file_location: str):
    self.__remote_s3_tfstate_service = RemoteS3TfStateService(
      file_location=file_location
    )

    super().__init__(
      tfstate_content=self.__remote_s3_tfstate_service.read_content()
    )



import os
from pathlib import Path
import markdown
from beautifultable import BeautifulTable

from tflens.model.tfstate_resource import TfStateResource

class TableHelper():

  def __init__(self, rows: list):
    self.table = BeautifulTable(maxwidth=200)
    self.table.columns.header = TfStateResource.get_str_headers()

    for row in rows or []:
      self.table.rows.append(row.get_str_row())

    self.table.rows.sort('module')

  def set_style(self, style):
    self.table.set_style(style)

  def print(self):
    print(self.table)

class MarkdownTableHelper(TableHelper):

  def __init__(self, rows: list):
    super().__init__(rows=rows)
    self.set_style(style=BeautifulTable.STYLE_MARKDOWN)

  def write_markdown_file(self):
    Path('.tflens').mkdir(parents=True, exist_ok=True)
    markdown_file_path = '.tflens/terraform.tfstate.md'

    with open(markdown_file_path, 'w') as markdown_file:
      print(self.table, file=markdown_file)

class HtmlTableHelper(TableHelper):

  def __init__(self, rows: list):
    super().__init__(rows=rows)
    self.set_style(style=BeautifulTable.STYLE_MARKDOWN)

  def write_html_file(self):
    Path('.tflens').mkdir(parents=True, exist_ok=True)
    html_file_path = '.tflens/terraform.tfstate.html'

    markdown_file_path = '.tflens/temp.md'
    with open(markdown_file_path, 'w') as markdown_file:
      print(self.table, file=markdown_file)

    markdown.markdownFromFile(
      input=markdown_file_path,
      output=html_file_path,
      encoding='utf8',
      extensions=['markdown.extensions.tables']
    )

    os.remove(markdown_file_path)



import argparse
from pathlib import Path

from tflens.controller.tfstate import (
  RemoteS3TfStateController,
  LocalTfStateController
)

parser = argparse.ArgumentParser(
  description='Terraform lens is a CLI tool that enables developers have a summarized view of tfstate resources.'
)

parser.add_argument('-f', '--file-location',
  type=str,
  action="store",
  dest="file_location",
  help="Defines the location (remote or local) of the tfstate file. \
    Mandatory if remote tfstate is selected. \
      If empty then use the current_folder/terraform.tfstate",
  default="")

parser.add_argument('-o', '--output',
  type=str,
  action="store",
  dest="output",
  help="Defines output type (markdown|html). If empty outputs in terminal",
  default="")

parser.add_argument('-r', '--remote',
  type=str,
  action="store",
  dest="remote",
  help="Defines if remote (s3) or local tfstate file. If empty local is used. \
    When remote is defined, you also need to specify --file-location with the tfstate location \
      according to the following pattern: bucket-name/tfstate-key",
  default="")

args = parser.parse_args()

ARGS_REMOTE = args.remote
ARGS_FILE_LOCATION = args.file_location
ARGS_OUTPUT = args.output

if not ARGS_FILE_LOCATION:
  ARGS_FILE_LOCATION = "{}/terraform.tfstate".format(Path().absolute())

def main():
  remote_router = {
    's3': RemoteS3TfStateController,
    '': LocalTfStateController
  }

  tfstate_controller = remote_router[ARGS_REMOTE](ARGS_FILE_LOCATION)

  output_router = {
    'markdown': tfstate_controller.create_markdown_file,
    'html': tfstate_controller.create_html_file,
    '': tfstate_controller.show_resources,
  }

  output_router[ARGS_OUTPUT]()

if __name__ == "__main__":
  main()



from tflens.model.tfstate_resource import TfStateResource

class TfState():

  def __init__(self, content: dict):
    super().__init__()
    self.__content = content
    self.__version = self.__content['version']
    self.__tf_version = self.__content['terraform_version']
    self.__serial = self.__content['serial']
    self.__lineage = self.__content['lineage']
    self.__outputs = self.__content['outputs']

    self.__get_resources(self.__content['resources'])

  def __get_resources(self, source_resources: list):
    resources = list()

    for resource in source_resources or []:
      temp_resource = TfStateResource(resource)
      resources.append(temp_resource)

    self.__resources = resources

  def get_resources_count(self):
    return len(self.__resources)

  def get_resources(self):
    return self.__resources



import json
from json.decoder import JSONDecodeError
import pathlib
import boto3

from botocore.exceptions import ClientError

from tflens.exception.exception import (
  CannotLoadLocalFile,
  CannotReadLocalFile,
  CannotLoadRemoteFile
)

class RemoteS3TfStateService():

  def __init__(self, file_location: str):
    self.__s3_client = boto3.client('s3')
    self.__bucket_name = file_location.split('/')[0]
    self.__file_s3_key = "/".join(file_location.split('/')[1:])

  def read_content(self):
    try:
      response = self.__s3_client.get_object(
        Bucket=self.__bucket_name,
        Key=self.__file_s3_key
      )
      print(response)

      return json.loads(response['Body'].read())

    except ClientError:
      raise CannotLoadRemoteFile

class LocalTfStateService():

  def __init__(self, file_location: str):
    self.__file_location = None

    try:
      path = pathlib.Path(file_location)
      with path.open():
        self.__file_location = file_location

    except OSError:
      raise CannotLoadLocalFile

  def read_content(self):
    try:
      with open(self.__file_location, 'r') as tfstate_file:
        return json.loads(tfstate_file.read())

    except JSONDecodeError:
      raise CannotReadLocalFile



import setuptools

with open("README.md", "r") as fh:
  long_description = fh.read()

setuptools.setup(
  name="tflens",
  version="1.0.1",
  author="Juan Manuel Ruiz Fernández",
  description="Terraform state viewer",
  keywords="terraform viewer state tfstate cli",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/neovasili/tflens",
  packages=setuptools.find_packages(include=[
    'tflens',
    'tflens.controller',
    'tflens.exception',
    'tflens.helper',
    'tflens.model',
    'tflens.service'
  ]),
  entry_points = {
    "console_scripts": ['tflens = tflens.__main__:main']
  },
  classifiers=[
    "Development Status :: 5 - Production/Stable",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3",
    "Topic :: Software Development :: Documentation",
    "Topic :: Terminals",
  ],
  python_requires='>=3.6',
)



class TfStateResource():

  def __init__(self, resource: dict):
    super().__init__()
    self.__resource = resource
    self.__module = None

    if 'module' in self.__resource:
      self.__module = self.__resource['module']

    self.__mode = self.__resource['mode']
    self.__type = self.__resource['type']
    self.__name = self.__resource['name']
    self.__provider = self.__resource['provider']
    self.__instances = self.__resource['instances']

  @classmethod
  def get_str_headers(cls):
    return ['provider', 'type', 'mode', 'name', 'module']

  def get_str_row(self):
    return [
      self.__provider,
      self.__type,
      self.__mode,
      self.__name,
      self.get_parent_module()
    ]

  def get_parent_module(self):
    return self.__module.split('.')[1] if self.__module else '(None)'

  def get_instances_count(self):
    return len(self.__instances)
